@@ -103,7 +103,7 @@ class AdsAPI(object):
         h = hmac.new(access_token, app_secret, hashlib.sha256)
         self.appsecret_proof = h.hexdigest()
 
-    def make_request(self, path, method, args=None, files=None, batch=False):
+    def make_request(self, path, method, args=None, files=None, batch=False, raw_path=False):
         """Makes a request against the Facebook Ads API endpoint."""
         args = dict(args or {})
 
@@ -118,11 +118,10 @@ class AdsAPI(object):
             args['access_token'] = self.access_token
         try:
             if method == 'GET':
-                url = '%s/%s?%s' % (FACEBOOK_API, path, urllib.urlencode(args))
-                print url
+                url = path if raw_path else '%s/%s?%s' % (FACEBOOK_API, path, urllib.urlencode(args))
                 f = urllib2.urlopen(url)
             elif method == 'POST':
-                url = '%s/%s' % (FACEBOOK_API, path)
+                url = path if raw_path else '%s/%s' % (FACEBOOK_API, path)
                 if files:
                     encoder = MultipartFormdataEncoder()
                     content_type, body = encoder.encode(args, files)
@@ -132,7 +131,7 @@ class AdsAPI(object):
                 else:
                     f = urllib2.urlopen(url, urllib.urlencode(args))
             elif method == 'DELETE':
-                url = '%s/%s?%s' % (FACEBOOK_API, path, urllib.urlencode(args))
+                url = path if raw_path else '%s/%s?%s' % (FACEBOOK_API, path, urllib.urlencode(args))
                 req = urllib2.Request(url)
                 req.get_method = lambda: 'DELETE'
                 f = urllib2.urlopen(req)
@@ -250,7 +249,10 @@ class AdsAPI(object):
     def get_adcampaigns_of_account(self, account_id, fields, batch=False):
         """Returns the fields of all ad sets from the given ad account."""
         path = 'act_%s/adcampaigns' % account_id
-        args = {'fields': fields}
+        args = {
+            'fields': fields,
+            'limit': self.DATA_LIMIT
+        }
         return self.make_request(path, 'GET', args, batch=batch)
 
     def get_adcampaigns(self, account_id, fields=None, batch=False):
@@ -303,12 +305,12 @@ class AdsAPI(object):
 
     def get_stats_by_adaccount(self, account_id, batch=False, start_time=None, end_time=None):
         """Returns the stats for a Facebook campaign."""
-        path = 'act_%s/adcampaignstats' % account_id
         args = {}
         if start_time:
             args['start_time'] = self.__parse_time(start_time)
         if end_time:
             args['end_time'] = self.__parse_time(end_time)
+        path = 'act_%s/adcampaignstats' % account_id
         return self.make_request(path, 'GET', args, batch=batch)
 
     # New API
@@ -316,7 +318,6 @@ class AdsAPI(object):
             self, campaign_group_id, fields=None, filters=None, batch=False,
             start_time=None, end_time=None):
         """Returns the stats for a Facebook campaign group."""
-        path = '%s/stats' % campaign_group_id
         args = {}
         if fields:
             args['fields'] = json.dumps(fields)
@@ -326,12 +327,12 @@ class AdsAPI(object):
             args['start_time'] = self.__parse_time(start_time)
         if end_time:
             args['start_time'] = self.__parse_time(end_time)
+        path = '%s/stats' % campaign_group_id
         return self.make_request(path, 'GET', args, batch=batch)
 
     def get_stats_by_adcampaign(self, account_id, campaign_ids=None,
                                 batch=False, start_time=None, end_time=None):
         """Returns the stats for a Facebook campaign by adcampaign."""
-        path = 'act_%s/adcampaignstats' % account_id
         args = {}
         if campaign_ids is not None:
             args['campaign_ids'] = json.dumps(campaign_ids)
@@ -339,13 +340,13 @@ class AdsAPI(object):
             args['start_time'] = self.__parse_time(start_time)
         if end_time:
             args['start_time'] = self.__parse_time(end_time)
+        path = 'act_%s/adcampaignstats' % account_id
         return self.make_request(path, 'GET', args, batch=batch)
 
     def get_stats_by_adgroup(
             self, account_id, adgroup_ids=None, batch=False,
             start_time=None, end_time=None):
         """Returns the stats for a Facebook campaign by adgroup."""
-        path = 'act_%s/adgroupstats' % account_id
         args = {}
         if adgroup_ids is not None:
             args['adgroup_ids'] = json.dumps(adgroup_ids)
@@ -353,6 +354,7 @@ class AdsAPI(object):
             args['start_time'] = self.__parse_time(start_time)
         if end_time:
             args['start_time'] = self.__parse_time(end_time)
+        path = 'act_%s/adgroupstats' % account_id
         return self.make_request(path, 'GET', args, batch=batch)
 
     # New API
@@ -436,8 +438,10 @@ class AdsAPI(object):
         path = 'act_%s/conversions' % account_id
         return self.make_request(path, 'GET', batch=batch)
 
-    def get_conversion_stats_by_adcampaign(self, account_id, campaign_ids=None,
-                                           include_deleted=False, batch=False):
+    def get_conversion_stats_by_adcampaign(
+            self, account_id, campaign_ids=None, include_deleted=False,
+            start_time=None, end_time=None, aggregate_days=None,
+            by_impression_time=True, batch=False):
         """Returns the conversions stats for all ad campaigns."""
         path = 'act_%s/adcampaignconversions' % account_id
         args = {}
@@ -445,6 +449,14 @@ class AdsAPI(object):
             args['campaign_ids'] = json.dumps(campaign_ids)
         if include_deleted is not None:
             args['include_deleted'] = include_deleted
+        if start_time is not None:
+            args['start_time'] = start_time
+        if end_time is not None:
+            args['end_time'] = end_time
+        if aggregate_days is not None:
+            args['aggregate_days'] = aggregate_days
+        if not by_impression_time:
+            args['by_impression_time'] = 'false'
         return self.make_request(path, 'GET', args, batch=batch)
 
     def get_conversion_stats_by_adgroup(self, account_id, adgroup_ids=None,
@@ -900,3 +912,13 @@ class AdsAPI(object):
                 raise Exception("Unknown __parse_time format for {0}".format(time_obj))
             return str(resp)
         return None
+
+    def __page_results(self, path, args, batch):
+        response = self.make_request(path, 'GET', args, batch=batch)
+        while True:
+            yield response
+            next_page = response.get('paging', {}).get('next', '')
+            if not next_page:
+                break
+            response = json.load(urllib2.urlopen(next_page))
+
